/***************************************************************************
 *   Copyright (C) 2004-2005 by Andrew Krause                              *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef GTKMM_MAIN_H
#define GTKMM_MAIN_H

// The motherload of includes for all other files. (Note: This file is included
// by most others in this code, so most Gtk files will already be included)
#include "editor.h"
#include "notebookmanager.h"
#include "project.h"
#include "template.h"
#include "variables.h"
#include "outputmanager.h"
#include "findtoolbar.h"
#include "gototoolbar.h"
#include "browser.h"

#include <gdkmm/screen.h>
#include <gtkmm/actiongroup.h>
#include <gdkmm/event.h>
#include <gtkmm/fixed.h>
#include <gtkmm/imagemenuitem.h>
#include <gtkmm/liststore.h>
#include <gtkmm/menu.h>
#include <gtkmm/menubar.h>
#include <gtkmm/paned.h>
#include <gtkmm/scrolledwindow.h>
#include <gtkmm/statusbar.h>
#include <gtkmm/toggleaction.h>
#include <gtkmm/toolbar.h>
#include <gtkmm/treestore.h>
#include <gtkmm/treeview.h>
#include <gtkmm/uimanager.h>
#include <pangomm/tabarray.h>
#include <sys/stat.h>

const char *const clipboard = "";

// Class to add column to the build output ListStore
class ModelSearchColumns : public Gtk::TreeModel::ColumnRecord
{
public:
  ModelSearchColumns()  { add(col_text); add(col_color); }
  Gtk::TreeModelColumn<Glib::ustring> col_text, col_color;
};

// Main class for openldev
class OpenLDev : public Gtk::Window
{
public:
  OpenLDev();                         
  ~OpenLDev() { }             

  // File commands
  void on_file_new();  
  void on_file_new_file();
  void on_file_open();                
  void on_file_save();                
  void on_file_saveas();
  void on_file_saveall();
  void on_file_revert();
  void on_file_quit();
  void close_project();
  void open_file(Glib::ustring);
  void open_project(Glib::ustring);
  void tree_view_selected();
  void tree_view_add_file();
  void tree_view_remove();

  // Edit commands
  void on_edit_undo();
  void on_edit_redo();
  void on_edit_cut();       
  void on_edit_copy();           
  void on_edit_paste();           
  void on_edit_selectall();
  void on_edit_deselect();
  void on_edit_find();
  void on_find_next();
  void on_edit_goto_line();
  void find_and_replace();
  void jump_to_line_number();
  void mark_found_text(Gtk::TextIter, Gtk::TextIter);
  void update_textview(Glib::RefPtr<Gtk::TextBuffer>);
  
  // View commands
  void on_view_toolbar();
  void on_view_statusbar();
  void on_view_fileviewer();
  void on_view_buildbox();
  void on_view_lnumbers();
  void close_toolbars();
  
  // Project building & compiling commands
  void on_compile();
  void on_build();
  void on_execute();
  void on_autogen();
  void on_configure();
  void on_clean();
  void on_stop();
  
  // Settings dialogs functions
  void on_settings_env();
  void on_settings_file_properties();
  void on_settings_filemanager();
  void on_settings_toolbar();
  void get_settings();
  void on_project_properties();
  
  // Help commands
  void on_help();
  void on_report_bug();
  void on_help_about();            
  
  // Initialization functions
  void populate_window();
  void initialize_objects();
  void populate_treeview();
  void populate_find_toolbar();
  void populate_goto_line_toolbar();
  void create_icon_factory();
  void create_action_group();
  void create_uimanager_string();
  void connect_signals();
  void create_menus_and_toolbar();
  void initialize_treeview();
  
  // Event managers
  void build_box_selected(GdkEventButton*);
  void search_box_selected(GdkEventButton*);
  bool on_textview_button_event(GdkEventButton*);
  void on_button_press_event_treeview(GdkEventButton*);
  void on_resize(GdkEvent*);
  void on_resize_pane(GdkEvent*);
  
  // Status bar control & syntax highlighting
  void update_status_bar (Gtk::TextIter, Glib::RefPtr<Gtk::TextBuffer::Mark>);
 
private:
  // Container Widgets
  Gtk::HPaned *hpaned;
  Gtk::VPaned *vpaned;
  Gtk::VBox *vbox;
  Gtk::HBox *hbox;
  OutputManager *output;
  FindToolbar *findtoolbar;
  GotoLineToolbar *gototoolbar;
  Browser *browser;

  // Many types of bars
  Gtk::Statusbar *status_bar, *status_bar2;
  Gtk::Widget *Menubar;
  Gtk::Toolbar *toolbar;
  Glib::RefPtr<Gtk::UIManager> uimanager;
  Glib::RefPtr<Gtk::ActionGroup> actgroup;
  Glib::RefPtr<Gtk::ToggleAction> toggle_toolbar, toggle_lnumbers, toggle_status;
  Glib::ustring ui_info;
  Gtk::Menu *menu_source_browser, *menu_textview;
  
  // Environment & project settings
  ProjectSettings *project;
  EnvironmentSettings *env;
  EditorProperties *editor;
  NotebookManager *noteman;
  ProjectTemplate *ptemplate;
  TemplateList *list_of_templates;
};

#endif
