/***************************************************************************
 *   Copyright (C) 2005 by Andrew Krause                                   *
 *   Based on gtksourceviewmm by Jae Jang                                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef SOURCEVIEW_H
#define SOURCEVIEW_H

#include <glibmm.h>
#include <gdkmm/pixbuf.h>
#include <gtkmm/textview.h>

#include "sourcebuffer.h"

typedef struct _GtkSourceView GtkSourceView;
typedef struct _GtkSourceViewClass GtkSourceViewClass;

namespace Gtk
{ class SourceView_Class; 
}

namespace Gtk
{
  class SourceView : public Gtk::TextView
  {
  public:
    typedef SourceView CppObjectType;
    typedef SourceView_Class CppClassType;
    typedef GtkSourceView BaseObjectType;
    typedef GtkSourceViewClass BaseClassType;

    SourceView();
    virtual ~SourceView();
  
    static GType get_type()      G_GNUC_CONST;
    static GType get_base_type() G_GNUC_CONST;

    GtkSourceView*       gobj()       { return reinterpret_cast<GtkSourceView*>(gobject_); }
    const GtkSourceView* gobj() const { return reinterpret_cast<GtkSourceView*>(gobject_); }

    void set_buffer(const Glib::RefPtr<SourceBuffer>& buffer);
    Glib::RefPtr<SourceBuffer> get_buffer();

    void set_show_line_numbers (bool);
    bool get_show_line_numbers ();
    void set_show_line_markers (bool);
    bool get_show_line_markers ();
    void set_tabs_width (int);
    int  get_tabs_width ();
    void set_auto_indent (bool);
    bool get_auto_indent ();
    void set_insert_spaces_instead_of_tabs (bool);
    bool get_insert_spaces_instead_of_tabs ();
    void set_show_margin (bool);
    bool get_show_margin ();
    void set_margin (int);
    int  get_margin ();
    void set_marker_pixbuf (const Glib::ustring&, const Glib::RefPtr<Gdk::Pixbuf>&);
    Glib::RefPtr<const Gdk::Pixbuf>  get_marker_pixbuf (const Glib::ustring&);

    void set_smart_home_end	(bool);
    bool get_smart_home_end	();

    Glib::SignalProxy0< void > signal_undo();
    Glib::SignalProxy0< void > signal_redo();

  private:
    friend class SourceView_Class;
    static CppClassType sourceview_class_;

    SourceView(const SourceView&);
    SourceView& operator=(const SourceView&);

  protected:
    explicit SourceView(const Glib::ConstructParams& construct_params);
    explicit SourceView(GtkSourceView* castitem);
  };
}

namespace Glib
{ Gtk::SourceView* wrap(GtkSourceView* object, bool take_copy = false);
}

#endif
